<?php
namespace nulib\db\pgsql;

use nulib\db\CapacitorChannel;
use nulib\db\CapacitorStorage;

class PgsqlStorage extends CapacitorStorage {
  const SERDATA_DEFINITION = "text";
  const SERSUM_DEFINITION = "varchar(40)";
  const SERTS_DEFINITION = "timestamp";

  function __construct($pgsql) {
    $this->pgsql = Pgsql::with($pgsql);
  }

  protected Pgsql $pgsql;

  function db(): Pgsql {
    return $this->pgsql;
  }

  const PRIMARY_KEY_DEFINITION = [
    "id_" => "serial primary key",
  ];

  function _getCreateSql(CapacitorChannel $channel): string {
    $query = new query($this->_createSql($channel));
    return self::format_sql($channel, $query->getSql());
  }

  protected function _afterCreate(CapacitorChannel $channel): void {
    $db = $this->pgsql;
    $db->exec([
      "create table if not exists",
      "table" => "_channels",
      "cols" => [
        "name" => "varchar primary key",
        "table_name" => "varchar",
        "class" => "varchar",
      ],
    ]);
    $db->exec([
      "insert",
      "into" => "_channels",
      "values" => [
        "name" => $channel->getName(),
        "table_name" => $channel->getTableName(),
        "class" => get_class($channel),
      ],
      "suffix" => "on conflict (name) do nothing",
    ]);
  }

  protected function _beforeReset(CapacitorChannel $channel): void {
    $this->pgsql->exec([
      "delete",
      "from" => "_channels",
      "where" => [
        "name" => $channel->getName(),
      ],
    ]);
  }

  function _exists(CapacitorChannel $channel): bool {
    $tableName = $channel->getTableName();
    if (($index = strpos($tableName, ".")) !== false) {
      $schemaName = substr($tableName, 0, $index);
      $tableName = substr($tableName, $index + 1);
    } else {
      $schemaName = "public";
    }
    return null !== $this->pgsql->get([
      "select tablename from pg_tables",
      "where" => [
        "schemaname" => $schemaName,
        "tablename" => $tableName,
      ],
    ]);
  }

  function close(): void {
    $this->pgsql->close();
  }
}
